# get_io_format converts incoming data to a standard IO format. The original
#   content is added to the output in the "original_message" field automatically.
def get_io_format:
    {
        "input": .,
        "output": {
            "original_message": . | tojson
        }
    }
;

# add_field adds a key-value pair to the metadata objects generated by get_io_format
# Takes an input and output filter as parameters
# Capable of adding data to any of the objects in the metadata JSON (input, output, subrule, transform_path, etc)
# This function should be used for adding all metadata to your output - It scrubs illegal characters like | and \n
# Example function call: add_field(.input.field1; .output.subject)
def add_field($input_field; output_field):

    # input_field validation check
    if
        $input_field
    then
        # Convert input_field value as Json text string
        if 
            ($input_field | type == "string")
        then        
            # Strip out control codes and extended characters from input field value. This includes characters like: [\b \t \n \u000b \f \r \u000e \u000f]
            ($input_field | explode | map(select(. > 31)) | implode) as $strip_control
            |
            # Retain the | removal to support regex parsing in SIEM
            if ($strip_control | contains("|")) then ($strip_control | split("|") | join ("-")) as $strip_pipe else $strip_control as $strip_pipe end
            |            
            # Cleanup completed
            output_field = $strip_pipe
        else
            output_field = $input_field
        end
    else
        .
    end
;

# flatten_array will flatten or fan out the pipeline, returning n values
#  where n is the number of array records in the log message. If there are no records
#  the input is returned unchanged
def flatten_array($log_msg_field):
    if $log_msg_field then
        if ($log_msg_field | length) > 0 then
            # flatten our records into one message each
            $log_msg_field[]
        else
            .
        end
    else
        .
    end
;


# transform will normalize the incoming log into the LogRhythm Schema
#   that can then be forwarded to the SIEM
def transform:
    # First, convert to IO format.
    get_io_format |

#    .input.raw = (.input.response) #|


   
    # beatname is a required field for Open Collector Regex to work in the SIEM.
    # for elastic beats, capture the beatname into our output
    # for other beats add a value to beatname code example: add_field("filebeat"; .output.beatname) |
    add_field(.input."@metadata".beat; .output.beatname) |
    add_field(.input.fullyqualifiedbeatname; .output.fullyqualifiedbeatname)|
    add_field(.input.response.type; .output.device_type)|

    # TODO: insert transformation logic here to build output

    # this filter produces the output object, for sending to SYSLOG output
    # This filter should be left in place in most cases
    .output
;
